"""
AUTOGENERATED by git post-commit hook. Note that these are always from the PREVIOUS commit, and so may be out of date by one log entry
"""
git_branch    = "master"
git_shorthash = "caf37cc"
git_longhash  = "caf37ccbb0791eea885c2044224e8d0fa2ddaa66"
git_last_tag  = "0.9.0.dev3"
